@@ -125,6 +125,7 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
 	int ret = 0, fd, included, printclean = 1;
 	const struct optstruct *opt;
 	const char *virname;
+        const char **virpp = &virname; 
 	struct stat sb;
 
     if((opt = optget(opts, "exclude"))->enabled) {
@@ -190,7 +191,16 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
     }
 
     if((ret = cl_scandesc(fd, &virname, &info.blocks, engine, options)) == CL_VIRUS) {
-	logg("~%s: %s FOUND\n", filename, virname);
+        if (options & CL_SCAN_ALLMATCHES) {                                                                               
+	    int i = 0;
+	    virpp = (const char **)*virpp; /* temp hack for allscan until API enhancement */
+	    virname = virpp[0];
+	    while (virpp[i])
+		logg("~%s: %s FOUND\n", filename, virpp[i++]);
+	    free((void *)virpp);
+	}
+	else
+	    logg("~%s: %s FOUND\n", filename, virname);
 	info.files++;
 	info.ifiles++;
 
@@ -319,6 +329,8 @@ static int scanstdin(const struct cl_engine *engine, const struct optstruct *opt
 	int ret;
 	unsigned int fsize = 0;
 	const char *virname, *tmpdir;
+	const char **virpp = &virname;
+
 	char *file, buff[FILEBUFF];
 	size_t bread;
 	FILE *fs;
@@ -358,7 +370,17 @@ static int scanstdin(const struct cl_engine *engine, const struct optstruct *opt
     info.rblocks += fsize / CL_COUNT_PRECISION;
 
     if((ret = cl_scanfile(file, &virname, &info.blocks, engine, options)) == CL_VIRUS) {
-	logg("stdin: %s FOUND\n", virname);
+        if (options & CL_SCAN_ALLMATCHES) {
+            int i = 0;
+            virpp = (const char **)*virpp; /* temp hack for scanall mode until api augmentation */
+            virname = virpp[0];
+            while (virpp[i])
+                logg("stdin: %s FOUND\n", virpp[i++]);
+            free((void *)virpp);
+        }
+	else
+	    logg("stdin: %s FOUND\n", virname);
+
 	info.ifiles++;
 
 	if(bell)
@@ -587,6 +609,9 @@ int scanmanager(const struct optstruct *opts)
     }
 
     /* set scan options */
+    if(optget(opts, "allmatch")->enabled)
+	options |= CL_SCAN_ALLMATCHES;
+
     if(optget(opts,"phishing-ssl")->enabled)
 	options |= CL_SCAN_PHISHING_BLOCKSSL;
 
