@@ -53,7 +53,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 67
+#define CL_FLEVEL 68
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -109,6 +109,8 @@ typedef struct bitset_tag
 /* internal clamav context */
 typedef struct cli_ctx_tag {
     const char **virname;
+    unsigned int num_viruses;         /* manages virname when CL_SCAN_ALLMATCHES == 1 */
+    unsigned int size_viruses;        /* manages virname when CL_SCAN_ALLMATCHES == 1 */
     unsigned long int *scanned;
     const struct cli_matcher *root;
     const struct cl_engine *engine;
@@ -314,6 +316,7 @@ extern int have_rar;
 #define DETECT_BROKEN	    (ctx->options & CL_SCAN_BLOCKBROKEN)
 #define BLOCK_MACROS	    (ctx->options & CL_SCAN_BLOCKMACROS)
 #define SCAN_STRUCTURED	    (ctx->options & CL_SCAN_STRUCTURED)
+#define SCAN_ALL            (ctx->options & CL_SCAN_ALLMATCHES)
 
 /* based on macros from A. Melnikoff */
 #define cbswap16(v) (((v & 0xff) << 8) | (((v) >> 8) & 0xff))
@@ -412,6 +415,10 @@ static inline void cli_writeint32(char *offset, uint32_t value)
 }
 #endif
 
+void cli_append_virus(cli_ctx *ctx, const char *virname);
+const char *cli_get_last_virus(const cli_ctx *ctx);
+const char *cli_get_last_virus_str(const cli_ctx *ctx);
+
 /* used by: spin, yc (C) aCaB */
 #define __SHIFTBITS(a) (sizeof(a)<<3)
 #define __SHIFTMASK(a) (__SHIFTBITS(a)-1)
